"""主检测器模块"""
import cv2
import os
from typing import List, Dict, Optional
from datetime import datetime
from tqdm import tqdm

from config.config import AppConfig, ModelConfig, DetectionConfig, OutputConfig
from core.image_processor import ImageProcessor
from core.model_inference import ONNXInferenceEngine
from core.post_processor import PostProcessor
from utils.file_manager import FileManager
from utils.report_generator import ReportGenerator


class UAVDetector:
    """UAV检测器主类"""
    
    def __init__(self, config: AppConfig = None, input_dir: str = None):
        # 初始化配置
        self.config = config or AppConfig()
        self.input_dir = input_dir
        
        # 统计信息
        self.image_count = 0
        self.detection_records = []
        
        # 初始化各个组件
        self._initialize_components()
    
    def _initialize_components(self):
        """初始化各个组件"""
        try:
            # 初始化图像处理器
            self.image_processor = ImageProcessor(self.config.model)
            
            # 初始化推理引擎
            self.inference_engine = ONNXInferenceEngine(self.config.model)
            
            # 初始化后处理器
            self.post_processor = PostProcessor(
                self.config.detection, 
                self.config.false_positive_features_path
            )
            
            # 初始化文件管理器
            output_dir = self.config.output.get_output_dir(self.input_dir)
            self.file_manager = FileManager(output_dir)
            
            print(f"检测器初始化成功")
            print(f"输出目录: {output_dir}")
            
        except Exception as e:
            raise RuntimeError(f"检测器初始化失败: {e}")
    
    def process_image(self, image_path: str) -> int:
        """处理单张图像
        
        Args:
            image_path: 图像文件路径
            
        Returns:
            检测到的目标数量
        """
        try:
            # 图像预处理
            input_data = self.image_processor.preprocess(image_path)
            if input_data is None:
                return 0
            
            # 模型推理
            detections = self.inference_engine.inference(input_data)
            if detections is None:
                return 0
            
            # 后处理
            original_image = cv2.imread(image_path)
            if original_image is None:
                return 0
            
            original_size = self.image_processor.get_original_size()
            valid_count, processed_img, detections_list = self.post_processor.postprocess(
                detections, original_image, original_size
            )
            
            # 保存结果
            self._save_results(image_path, processed_img, detections_list, valid_count)
            
            # 更新统计
            self.image_count += 1
            
            return valid_count
            
        except Exception as e:
            print(f"处理图像 {image_path} 时出错: {e}")
            return 0
    
    def _save_results(self, image_path: str, processed_img, detections_list: List[Dict], valid_count: int):
        """保存处理结果"""
        # 保存检测图像
        output_image_path = self.file_manager.save_detection_image(
            processed_img, image_path, 
            self.config.output.save_empty, 
            valid_count > 0
        )
        
        # 保存目标ROI
        for i, detection in enumerate(detections_list):
            self.file_manager.save_target_roi(
                detection['roi'], image_path, i
            )
        
        # 记录检测信息
        record = {
            'image_path': image_path,
            'has_detection': valid_count > 0,
            'detection_count': valid_count,
            'timestamp': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'output_image_path': output_image_path
        }
        
        if valid_count > 0:
            record['detections'] = detections_list
        
        self.detection_records.append(record)
    
    def process_batch(self, input_path: str) -> Dict[str, int]:
        """批量处理图像
        
        Args:
            input_path: 输入路径（文件或目录）
            
        Returns:
            处理结果统计
        """
        try:
            # 获取图像文件列表
            image_files = self.file_manager.get_image_files(input_path)
            
            total_detections = 0
            processed_images = 0
            
            print(f"开始处理 {len(image_files)} 张图像...")
            
            # 处理每张图像
            for image_file in tqdm(image_files, desc='Processing images'):
                detections = self.process_image(image_file)
                total_detections += detections
                processed_images += 1
                
                if detections > 0:
                    print(f'处理 {os.path.basename(image_file)} 完成，检测到 {detections} 个目标')
            
            # 生成报告
            self._generate_reports()
            
            # 清理空目录
            self.file_manager.cleanup_empty_directories()
            
            return {
                'processed_images': processed_images,
                'total_detections': total_detections,
                'images_with_detections': sum(1 for r in self.detection_records if r['has_detection'])
            }
            
        except Exception as e:
            print(f"批量处理失败: {e}")
            return {'processed_images': 0, 'total_detections': 0, 'images_with_detections': 0}
    
    def _generate_reports(self):
        """生成检测报告"""
        try:
            report_generator = ReportGenerator(self)
            output_paths = self.file_manager.get_output_paths()
            
            # 生成CSV报告
            report_generator.generate_csv(output_paths['csv_report'])
            print(f'CSV报告已生成: {output_paths["csv_report"]}')
            
            # 生成Excel报告
            report_generator.generate_excel(output_paths['excel_report'])
            print(f'Excel报告已生成: {output_paths["excel_report"]}')
            
        except Exception as e:
            print(f"生成报告时出错: {e}")
    
    def get_statistics(self) -> Dict:
        """获取检测统计信息"""
        total_detections = sum(r.get('detection_count', 0) for r in self.detection_records)
        images_with_detections = sum(1 for r in self.detection_records if r['has_detection'])
        
        return {
            'total_images': self.image_count,
            'total_detections': total_detections,
            'images_with_detections': images_with_detections,
            'detection_rate': images_with_detections / self.image_count if self.image_count > 0 else 0,
            'avg_detections_per_image': total_detections / self.image_count if self.image_count > 0 else 0
        }
    
    def get_model_info(self) -> Dict:
        """获取模型信息"""
        model_info = self.inference_engine.get_model_info()
        model_info.update({
            'confidence_threshold': self.config.detection.confidence_threshold,
            'iou_threshold': self.config.detection.iou_threshold,
            'max_bbox_ratio': self.config.detection.max_bbox_ratio,
            'class_names': self.config.detection.class_names
        })
        return model_info
    
    @property
    def model_version(self) -> str:
        """获取模型版本"""
        return os.path.basename(self.config.model.model_path).split('.')[0]
    
    @property
    def threshold(self) -> float:
        """获取置信度阈值（兼容性属性）"""
        return self.config.detection.confidence_threshold
    
    @property
    def output_dir(self) -> str:
        """获取输出目录（兼容性属性）"""
        return self.file_manager.output_dir