"""模型推理模块"""
import onnxruntime as ort
import numpy as np
from typing import Optional
from config.config import ModelConfig


class ONNXInferenceEngine:
    """ONNX模型推理引擎"""
    
    def __init__(self, config: ModelConfig):
        self.config = config
        self.model_path = config.model_path
        self.session = None
        self.input_name = None
        self.output_name = None
        
        self._initialize_session()
    
    def _initialize_session(self):
        """初始化ONNX推理会话"""
        try:
            # 配置会话选项
            session_options = ort.SessionOptions()
            session_options.log_severity_level = 3  # 减少日志输出
            
            # 创建推理会话
            self.session = ort.InferenceSession(
                self.model_path, 
                session_options, 
                providers=self.config.providers
            )
            
            # 获取输入输出名称
            self.input_name = self.session.get_inputs()[0].name
            self.output_name = self.session.get_outputs()[0].name
            
            print(f"模型加载成功: {self.model_path}")
            print(f"使用的执行提供者: {self.session.get_providers()}")
            
        except Exception as e:
            raise RuntimeError(f"初始化ONNX模型失败: {e}")
    
    def inference(self, input_data: np.ndarray) -> Optional[np.ndarray]:
        """执行模型推理
        
        Args:
            input_data: 预处理后的输入数据
            
        Returns:
            模型推理结果
        """
        if self.session is None:
            raise RuntimeError("模型未正确初始化")
        
        try:
            # 准备输入数据
            scale_factor = np.array([[1, 1]], dtype=np.float32)
            
            # 执行推理
            outputs = self.session.run(
                [self.output_name],
                {
                    self.input_name: input_data,
                    'scale_factor': scale_factor
                }
            )
            
            return outputs[0]
            
        except Exception as e:
            print(f"模型推理失败: {e}")
            return None
    
    def get_model_info(self) -> dict:
        """获取模型信息"""
        if self.session is None:
            return {}
        
        return {
            'model_path': self.model_path,
            'input_name': self.input_name,
            'output_name': self.output_name,
            'providers': self.session.get_providers(),
            'input_shape': self.session.get_inputs()[0].shape,
            'output_shape': self.session.get_outputs()[0].shape
        }