"""后处理模块"""
import cv2
import numpy as np
import pickle
import os
from typing import List, Dict, Tuple, Optional
from config.config import DetectionConfig


class PostProcessor:
    """检测结果后处理器"""
    
    def __init__(self, config: DetectionConfig, false_positive_features_path: str = None):
        self.config = config
        self.confidence_threshold = config.confidence_threshold
        self.iou_threshold = config.iou_threshold
        self.max_bbox_ratio = config.max_bbox_ratio
        self.class_names = config.class_names
        
        # 加载误报特征库
        self.false_positive_features = self._load_false_positive_features(false_positive_features_path)
    
    def _load_false_positive_features(self, features_path: str) -> dict:
        """加载误报特征库"""
        if not features_path:
            features_path = os.path.join(os.path.dirname(__file__), 'false_positive_features.pkl')
        
        try:
            with open(features_path, 'rb') as f:
                features = pickle.load(f)
            print(f"误报特征库加载成功: {features_path}")
            return features
        except FileNotFoundError:
            print(f"未找到误报特征库文件: {features_path}，跳过加载")
            return {}
        except Exception as e:
            print(f"加载误报特征库失败: {e}")
            return {}
    
    def nms(self, boxes: np.ndarray, scores: np.ndarray) -> List[int]:
        """非极大值抑制
        
        Args:
            boxes: 检测框坐标 [N, 4] (x1, y1, x2, y2)
            scores: 置信度分数 [N, 1]
            
        Returns:
            保留的检测框索引列表
        """
        if len(boxes) == 0:
            return []
        
        x1 = boxes[:, 0]
        y1 = boxes[:, 1]
        x2 = boxes[:, 2]
        y2 = boxes[:, 3]
        
        areas = (x2 - x1 + 1) * (y2 - y1 + 1)
        order = scores.flatten().argsort()[::-1]
        keep = []
        
        while order.size > 0:
            i = order[0]
            keep.append(i)
            
            if order.size == 1:
                break
            
            xx1 = np.maximum(x1[i], x1[order[1:]])
            yy1 = np.maximum(y1[i], y1[order[1:]])
            xx2 = np.minimum(x2[i], x2[order[1:]])
            yy2 = np.minimum(y2[i], y2[order[1:]])
            
            w = np.maximum(0.0, xx2 - xx1 + 1)
            h = np.maximum(0.0, yy2 - yy1 + 1)
            inter = w * h
            
            ovr = inter / (areas[i] + areas[order[1:]] - inter)
            inds = np.where(ovr <= self.iou_threshold)[0]
            order = order[inds + 1]
        
        return keep
    
    def _is_false_positive(self, roi: np.ndarray) -> bool:
        """使用ORB特征匹配验证是否误报
        
        Args:
            roi: 检测区域图像
            
        Returns:
            是否为误报
        """
        if not self.false_positive_features or roi.size == 0:
            return False
        
        try:
            # 初始化特征检测器
            detector = cv2.ORB_create()
            _, des = detector.detectAndCompute(roi, None)
            
            if des is None:
                return False
            
            # 与特征库进行匹配
            for fp_feature in self.false_positive_features.values():
                if fp_feature.get('features') is None:
                    continue
                
                # 使用FLANN匹配器
                try:
                    flann = cv2.FlannBasedMatcher(
                        dict(algorithm=6, table_number=6), 
                        dict()
                    )
                    matches = flann.knnMatch(des, fp_feature['features'], k=2)
                    
                    if not matches:
                        continue
                    
                    # 计算优质匹配
                    good_matches = []
                    for match_group in matches:
                        if len(match_group) < 2:
                            continue
                        m, n = match_group
                        if m.distance < 0.7 * n.distance:
                            good_matches.append(m)
                    
                    # 如果优质匹配数量超过阈值，认为是误报
                    if len(good_matches) > 15:
                        return True
                        
                except Exception as e:
                    print(f"特征匹配过程中出错: {e}")
                    continue
            
            return False
            
        except Exception as e:
            print(f"误报检测过程中出错: {e}")
            return False
    
    def postprocess(self, detections: np.ndarray, original_image: np.ndarray, 
                   original_size: Tuple[int, int]) -> Tuple[int, np.ndarray, List[Dict]]:
        """后处理检测结果
        
        Args:
            detections: 模型输出的检测结果
            original_image: 原始图像
            original_size: 原始图像尺寸 (width, height)
            
        Returns:
            (有效检测数量, 处理后的图像, 检测结果列表)
        """
        orig_w, orig_h = original_size
        img_out = original_image.copy()
        detections_list = []
        
        # 过滤低置信度检测
        keep_idx = (detections[:, 1] > self.confidence_threshold)
        detections = detections[keep_idx]
        
        if len(detections) == 0:
            return 0, img_out, detections_list
        
        # 坐标转换：从模型输入尺寸转换到原始图像尺寸
        ratio_w = orig_w / self.config.input_size[0] if hasattr(self.config, 'input_size') else orig_w / 640
        ratio_h = orig_h / self.config.input_size[1] if hasattr(self.config, 'input_size') else orig_h / 640
        
        detections[:, 2:6] *= np.array([ratio_w, ratio_h, ratio_w, ratio_h])
        
        # NMS处理
        keep = self.nms(detections[:, 2:6], detections[:, 1:2])
        
        valid_detections = 0
        
        for idx in keep:
            class_id = int(detections[idx, 0])
            confidence = detections[idx, 1]
            x1, y1, x2, y2 = detections[idx, 2:6].astype(int)
            
            # 确保坐标在图像范围内
            x1 = max(0, min(x1, orig_w - 1))
            y1 = max(0, min(y1, orig_h - 1))
            x2 = max(x1 + 1, min(x2, orig_w))
            y2 = max(y1 + 1, min(y2, orig_h))
            
            # 计算检测框面积比例
            bbox_area = (x2 - x1) * (y2 - y1)
            image_area = orig_w * orig_h
            if bbox_area / image_area > self.max_bbox_ratio:
                continue
            
            # 截取ROI区域进行误报验证
            roi = original_image[y1:y2, x1:x2]
            if roi.size == 0 or self._is_false_positive(roi):
                continue
            
            # 绘制检测框和标签
            class_name = self.class_names.get(class_id, f'Class_{class_id}')
            label = f'{class_name} {confidence:.2f}'
            
            cv2.rectangle(img_out, (x1, y1), (x2, y2), (255, 0, 0), 4)
            cv2.putText(img_out, label, (x1, y1 - 10), 
                       cv2.FONT_HERSHEY_SIMPLEX, 0.8, (0, 0, 255), 2)
            
            # 记录检测结果
            detections_list.append({
                'class_id': class_id,
                'class_name': class_name,
                'confidence': float(confidence),
                'bbox': [x1, y1, x2, y2],
                'roi': roi,
                'orig_w': orig_w,
                'orig_h': orig_h
            })
            
            valid_detections += 1
        
        return valid_detections, img_out, detections_list