"""文件管理模块"""
import os
import cv2
from glob import glob
from typing import List, Optional, Generator
from datetime import datetime


class FileManager:
    """文件管理器"""
    
    def __init__(self, output_dir: str):
        self.output_dir = output_dir
        self.targets_dir = os.path.join(output_dir, 'targets')
        self.images_dir = os.path.join(output_dir, 'imgs')
        
        # 创建输出目录
        self._create_directories()
    
    def _create_directories(self):
        """创建必要的目录"""
        os.makedirs(self.output_dir, exist_ok=True)
        os.makedirs(self.targets_dir, exist_ok=True)
        os.makedirs(self.images_dir, exist_ok=True)
    
    @staticmethod
    def get_image_files(input_path: str) -> List[str]:
        """获取图像文件列表
        
        Args:
            input_path: 输入路径（文件或目录）
            
        Returns:
            图像文件路径列表
        """
        supported_formats = ('.jpg', '.jpeg', '.png', '.bmp', '.tiff', '.tif')
        
        if os.path.isfile(input_path):
            if input_path.lower().endswith(supported_formats):
                return [input_path]
            else:
                raise ValueError(f"不支持的图像格式: {input_path}")
        
        elif os.path.isdir(input_path):
            image_files = []
            for root, _, files in os.walk(input_path):
                for file in files:
                    if file.lower().endswith(supported_formats):
                        image_files.append(os.path.join(root, file))
            
            if not image_files:
                raise ValueError(f"在目录 {input_path} 中未找到支持的图像文件")
            
            return sorted(image_files)
        
        else:
            raise ValueError(f"输入路径不存在: {input_path}")
    
    def save_detection_image(self, image: any, image_path: str, save_empty: bool = False, 
                           has_detection: bool = False) -> Optional[str]:
        """保存检测结果图像
        
        Args:
            image: 处理后的图像
            image_path: 原始图像路径
            save_empty: 是否保存无检测结果的图像
            has_detection: 是否有检测结果
            
        Returns:
            保存的文件路径，如果未保存则返回None
        """
        if has_detection or save_empty:
            filename = f"{os.path.splitext(os.path.basename(image_path))[0]}.jpg"
            output_path = os.path.join(self.images_dir, filename)
            cv2.imwrite(output_path, image)
            return output_path
        return None
    
    def save_target_roi(self, roi: any, image_path: str, detection_index: int) -> str:
        """保存检测到的目标ROI
        
        Args:
            roi: 目标区域图像
            image_path: 原始图像路径
            detection_index: 检测索引
            
        Returns:
            保存的ROI文件路径
        """
        base_name = os.path.splitext(os.path.basename(image_path))[0]
        target_filename = f"{base_name}_{detection_index}.jpg"
        target_path = os.path.join(self.targets_dir, target_filename)
        cv2.imwrite(target_path, roi)
        return target_path
    
    @staticmethod
    def select_input_directory() -> Optional[str]:
        """图形界面选择输入目录
        
        Returns:
            选择的目录路径，如果取消则返回None
        """
        try:
            import tkinter as tk
            from tkinter import filedialog
            
            root = tk.Tk()
            root.withdraw()  # 隐藏主窗口
            
            folder_path = filedialog.askdirectory(title='选择输入目录')
            root.destroy()
            
            return folder_path if folder_path else None
            
        except ImportError:
            print("tkinter模块不可用，无法使用图形界面")
            return None
        except Exception as e:
            print(f"选择目录时出错: {e}")
            return None
    
    def get_output_paths(self) -> dict:
        """获取输出路径信息
        
        Returns:
            包含各种输出路径的字典
        """
        return {
            'output_dir': self.output_dir,
            'targets_dir': self.targets_dir,
            'images_dir': self.images_dir,
            'csv_report': os.path.join(self.output_dir, 'detection_report.csv'),
            'excel_report': os.path.join(self.output_dir, 'detection_report.xlsx')
        }
    
    def cleanup_empty_directories(self):
        """清理空目录"""
        for dir_path in [self.targets_dir, self.images_dir]:
            try:
                if os.path.exists(dir_path) and not os.listdir(dir_path):
                    os.rmdir(dir_path)
                    print(f"已删除空目录: {dir_path}")
            except OSError as e:
                print(f"删除目录 {dir_path} 时出错: {e}")